import { Engine, Rule } from 'json-rules-engine';
import { resolveEventPathParam, removeErrorFromControllers, extractTemplatePath, filterOutErrorsByType } from './rules-engine-form-helpers';
import { switchMap, from } from 'rxjs';
/*
we make a minor change in the lib of "JSON-RULE-ENGINE"
 that give the ability to pass the angular form object reference.
this is our edit list files:
node_modules\json-rules-engine\dist\rule.js line 266 ,283
node_modules\json-rules-engine\dist\condition.js 136 ,123
node_modules\json-rules-engine\dist\operator.js 43

*/
function removeErrors(event, f) {
    // check for previous errors success that need to be removed in this check
    let controlPaths = resolveEventPathParam(event.params.formJsonPath, event.params.formPath, true, event.params.errorAtIndex);
    removeErrorFromControllers(controlPaths, f, event.type);
}
function addErrors(event, f) {
    let controlPathsErrorsTurnOff = [];
    // step 1 - turn off validation that marked as false
    if (event.params.formJsonPath) {
        controlPathsErrorsTurnOff = extractTemplatePath(event.params.formJsonPath, false, event.params.errorAtIndex);
    }
    removeErrorFromControllers(controlPathsErrorsTurnOff, f, event.type);
    let controlPathsWithErrorTurnOn = resolveEventPathParam(event.params.formJsonPath, event.params.formPath, true, event.params.errorAtIndex);
    controlPathsWithErrorTurnOn.forEach(controlPath => {
        const control = f.get(controlPath);
        if (control === null) {
            console.error('not found control', controlPath);
            return;
        }
        ;
        const errors = control.errors ? filterOutErrorsByType(control.errors, event.type) : {};
        console.log('previous errors on control to keep on', errors);
        // step 2 -  keep previous errors and add the new errors
        control.setErrors({
            ...control.errors,
            [event.type]: true
        });
    });
}
export function initEngine(rules, f) {
    const engine = new Engine(rules.map(rule => new Rule({
        ...rule,
        onFailure: function (event, almanac, ruleResult) {
            /* when the rule fail we
                1- as practice we not add errors in 'onFailure' event!
                    we add errors only on 'onSuccess' event
                2- we need to check for previous errors success that need to be removed in this check
        
            */
            removeErrors(event, f);
        },
        onSuccess: function (event, almanac, ruleResult) {
            /* when the rule success we
              1- if the rule success for some array items in the same time it my fail for other array items
                so first we need to turn of validation that was previous success but in this cycle were failed
              2- we need to turn on errors success items
          */
            addErrors(event, f);
        }
    })), {
        allowUndefinedFacts: true
    });
    return engine;
}
export function rulesEngineRun(engine, f) {
    console.log('form raw data', f.value);
    const data = f.value;
    // 'main' will be the key inside the rule engine for the form  root
    return engine.run({ 'main': data }).then(() => engine.stop());
}
export function createEngineAndOperator(rules, form) {
    const engine = initEngine(rules, form);
    return {
        engine,
        operatorEngine: EngineOperator(engine, form)
    };
}
export function EngineOperator(engine, form) {
    return switchMap(() => from(rulesEngineRun(engine, form)));
}
//# sourceMappingURL=json-rules-engine-form.js.map